/*
	(c) 2021 Dmitry Grinberg   https://dmitry.gr
	Non-commercial use only OR licensing@dmitry.gr
*/

#ifndef _LK_401_KEYBOARD_H_
#define _LK_401_KEYBOARD_H_

#include <stdbool.h>
#include <stdint.h>

enum Lk401Key;
typedef void (*Lk401TxF)(uint8_t chr);

void lk401init(Lk401TxF txF);
void lk401RxByte(uint8_t byte);

void lk401KeyState(enum Lk401Key key, bool down);

enum Lk401Key {
	//group 10 keys
	Lk201_F1 = 0x56,
	Lk201_F2,
	Lk201_F3,
	Lk201_F4,
	Lk201_F5,
	Lk201_GRP10_first = Lk201_F1,
	Lk201_GRP10_last = Lk201_F5,
	
	//group 11 keys
	Lk201_F6 = 0x64,
	Lk201_F7,
	Lk201_F8,
	Lk201_F9,
	Lk201_F10,
	Lk201_GRP11_first = Lk201_F6,
	Lk201_GRP11_last = Lk201_F10,
	
	//group 12 keys
	Lk201_F11 = 0x71,
	Lk201_F12,
	Lk201_F13,
	Lk201_F14,
	Lk201_GRP12_first = Lk201_F11,
	Lk201_GRP12_last = Lk201_F14,
	
	//group 13 keys
	Lk201_HELP = 0x7c,		//where pint screen would be
	Lk201_MENU,				//also seen as "DO" key which occupies space where both scrolllock and pause/break key are)
	Lk201_GRP13_first = Lk201_HELP,
	Lk201_GRP13_last = Lk201_MENU,
	
	//group 14 keys:
	Lk201_F17 = 0x80,
	Lk201_F18,
	Lk201_F19,
	Lk201_F20,
	Lk201_GRP14_first = Lk201_F17,
	Lk201_GRP14_last = Lk201_F20,
	
	//group 9 keys (where PCs have insert, home, pgup, delete, end, pgdown, keys here in same order)
	Lk201_FIND = 0x8a,
	Lk201_INSERT,
	Lk201_REMOVE,
	Lk201_SELECT,
	Lk201_PREV_SCREEN,
	Lk201_NEXT_SCREEN,
	Lk201_GRP9_first = Lk201_FIND,
	Lk201_GRP9_last = Lk201_NEXT_SCREEN,

	//group 2 keys (keypad)
	Lk201_KP_0 = 0x92,
	Lk201_KP_PERIOD = 0x94,
	Lk201_KP_ENTER,
	Lk201_KP_1,
	Lk201_KP_2,
	Lk201_KP_3,
	Lk201_KP_4,
	Lk201_KP_5,
	Lk201_KP_6,
	Lk201_KP_COMMA,
	Lk201_KP_7,
	Lk201_KP_8,
	Lk201_KP_9,
	Lk201_KP_MINUS,
	Lk201_KP_PF1,		//PF1..PF3 keys above numpad
	Lk201_KP_PF2,
	Lk201_KP_PF3,
	Lk201_KP_PF4,
	Lk201_GRP2_first = Lk201_KP_0,
	Lk201_GRP2_last = Lk201_KP_PF4,
	
	//group 7 keys
	Lk201_LEFT = 0xa7,
	Lk201_RIGHT,
	Lk201_GRP7_first = Lk201_LEFT,
	Lk201_GRP7_last = Lk201_RIGHT,
	
	//group 8 keys
	Lk201_DOWN = 0xa9,
	Lk201_UP,
	Lk401_RSHIFT,
	Lk401_LALT,
	Lk401_RCOMPOSE,
	Lk201_GRP8_first = Lk201_DOWN,
	Lk201_GRP8_last = Lk401_RCOMPOSE,
	
	//group 6 keys
	Lk201_LSHIFT = 0xae,
	Lk201_CTRL,				//only exists on left
	Lk201_GRP6_first = Lk201_LSHIFT,
	Lk201_GRP6_last = Lk201_CTRL,
	
	//group 5 keys
	Lk201_LOCK = 0xb0,		//capslock
	Lk201_META,				//XXX which is this?
	Lk401_RALT,
	Lk201_GRP5_first = Lk201_LOCK,
	Lk201_GRP5_last = Lk401_RALT,
	
	//group 3 keys
	Lk201_BACKSPACE = 0xbc,
	Lk201_GRP3_first = Lk201_BACKSPACE,
	Lk201_GRP3_last = Lk201_BACKSPACE,

	//group 4 keys
	Lk201_ENTER = 0xbd,
	Lk201_TAB,
	Lk201_GRP4_first = Lk201_ENTER,
	Lk201_GRP4_last = Lk201_TAB,

	//group 1 keys
	Lk201_Tilde = 0xbf,
	Lk201_1,				//not numpad
	Lk201_Q,
	Lk201_A,
	Lk201_Z,
	Lk201_2=0xc5,
	Lk201_W,
	Lk201_S,
	Lk201_X,
	Lk201_LEFT_RIGHT,		//key with both left and right arrows between Lshift and Z
	Lk201_3 = 0xcb,
	Lk201_E,
	Lk201_D,
	Lk201_C,
	Lk201_4 = 0xd0,
	Lk201_R,
	Lk201_F,
	Lk201_V,
	Lk201_SPACE,
	Lk201_5 = 0xd6,
	Lk201_T,
	Lk201_G,
	Lk201_B,
	Lk201_6 = 0xdb,
	Lk201_Y,
	Lk201_H,
	Lk201_N,
	Lk201_7 = 0xe0,
	Lk201_U,
	Lk201_J,
	Lk201_M,
	Lk201_8 = 0xe5,
	Lk201_I,
	Lk201_K,
	Lk201_COMMA,
	Lk201_9 = 0xea,
	Lk201_O,
	Lk201_L,
	Lk201_PERIOD,
	Lk201_0 = 0xef,
	Lk201_P,
	Lk201_SEMICOLON = 0xf2,
	Lk201_QUESTION,
	Lk201_PLUS = 0xf5,
	Lk201_RBRACE,
	Lk201_PIPE,
	Lk201_MINUS = 0xf9,
	Lk201_LBRACE,
	Lk201_QOUTE,
	Lk201_GRP1_first = Lk201_Tilde,
	Lk201_GRP1_last = Lk201_QOUTE,
};


#endif


