/*
	(c) 2021 Dmitry Grinberg   https://dmitry.gr
	Non-commercial use only OR licensing@dmitry.gr
*/

#include <stdio.h>
#include "lk401.h"


//more: https://www.netbsd.org/docs/Hardware/Machines/DEC/lk201_keycodes.html
//too much per-key bookkeeping is needed to make autorepeat work as promised, so i don't...fuck it

enum KeyboardState {
	KeyboardIdle,
	KeyboardWaitLedData,
	KeyboardWaitClickVolumeByte,
	KeyboardWaitBellVolumeByte,
	KeyboardWaitAutorepeatGroupNum,
	KeyboardWaitAutorepeatConfigByte1,
	KeyboardWaitAutorepeatConfigByte2,
	KeyboardLocked,
};

enum KeyGroupMode {
	KeyGroupModeDown,		//only send keydown
	KeyGroupModeAutodown,	//send down and autorepeat
	KeyGroupModeUnused_2,	//not used
	KeyGroupModeUpDown,		//send up and down
};

struct AutorepeatGroupConfig {
	uint8_t timeout;	//wait till autorepeat in 5ms increments
	uint8_t rate;		//in Hz. valid range: 12..124
};

struct KeyGroup {
	uint8_t autorepeatGroup	: 2;
	uint8_t mode			: 2;	//enum KeyGroupMode
	uint8_t nKeysDown;
};

static Lk401TxF mTxF;
static enum KeyboardState mState;
static struct AutorepeatGroupConfig mAutorepeatGroups[4];
static struct KeyGroup mKeyGroups[14];
static uint8_t mCurGroup;
static uint8_t mCurStates[64];	//simpler to have all bits than to be picky
static bool mGlobalAutorepeatOn;

void lk401init(Lk401TxF txF)
{
	mTxF = txF;
}

void lk401RxByte(uint8_t byte)
{
	uint_fast8_t i;
	
	switch (mState) {
		case KeyboardIdle:
			
			switch (byte) {
				case 0x11:	//leds off
				case 0x13:	//leds on
					mState = KeyboardWaitLedData;
					break;
					
				case 0x1b:	//key click enable and set volume
					mState = KeyboardWaitClickVolumeByte;
					break;
				
				case 0x23:	//bell enable and set volume
					mState = KeyboardWaitBellVolumeByte;
					break;
				
				case 0xe1:	//disable autorepeat
					mGlobalAutorepeatOn = false;
					break;
					
				case 0xe3:	//enable autorepeat
					mGlobalAutorepeatOn = true;
					break;
				
				case 0xbb:	//enable key blick for control key
				case 0xb9:	//diable key clicks for contorl key
				case 0x99:	//disable key clicks
				case 0xa1:	//bell disable
				case 0xd1:	//temporary disable autorepeat
				case 0xd3:	//set defaults
				case 0x8b:	//unlock if not locked - no-op
				case 0xe9:	//enable LK401 mode (keys that are on LK401 but not on LK201) will now work (for compat by default they do not)
					break;
				
				case 0x89:	//lock keyboard
					mState = KeyboardLocked;
					mTxF(0xb7);	//ack it
					break;
				
				case 0x9f:	//make a key click sound
					break;
				
				case 0xa7:	//ring bell
					break;
				
				case 0xd9:	//switch all keys programmed to just send keydown to also do autorepeat
					for (i = 0; i < 14; i++) {
						
						if (mKeyGroups[i].mode == KeyGroupModeAutodown)
							mKeyGroups[i].mode = KeyGroupModeDown;
					}
					break;
				
			#define MODECHANGES_FOR_GRP(param, x)	((param ? 0x00 : 0x80) + ((x) << 3)): 		\
													case ((param ? 0x02 : 0x82) + ((x) << 3)):	\
													case ((param ? 0x06 : 0x86) + ((x) << 3))
			#define EXTRACT_GROUP_NUM(x)			(((x) >> 3) & 15)
			
			
				case MODECHANGES_FOR_GRP(0, 1):
				case MODECHANGES_FOR_GRP(0, 2):
				case MODECHANGES_FOR_GRP(0, 3):
				case MODECHANGES_FOR_GRP(0, 4):
				case MODECHANGES_FOR_GRP(0, 5):
				case MODECHANGES_FOR_GRP(0, 6):
				case MODECHANGES_FOR_GRP(0, 7):
				case MODECHANGES_FOR_GRP(0, 8):
				case MODECHANGES_FOR_GRP(0, 9):
				case MODECHANGES_FOR_GRP(0, 10):
				case MODECHANGES_FOR_GRP(0, 11):
				case MODECHANGES_FOR_GRP(0, 12):
				case MODECHANGES_FOR_GRP(0, 13):
				case MODECHANGES_FOR_GRP(0, 14):
				case MODECHANGES_FOR_GRP(1, 1):
				case MODECHANGES_FOR_GRP(1, 2):
				case MODECHANGES_FOR_GRP(1, 3):
				case MODECHANGES_FOR_GRP(1, 4):
				case MODECHANGES_FOR_GRP(1, 5):
				case MODECHANGES_FOR_GRP(1, 6):
				case MODECHANGES_FOR_GRP(1, 7):
				case MODECHANGES_FOR_GRP(1, 8):
				case MODECHANGES_FOR_GRP(1, 9):
				case MODECHANGES_FOR_GRP(1, 10):
				case MODECHANGES_FOR_GRP(1, 11):
				case MODECHANGES_FOR_GRP(1, 12):
				case MODECHANGES_FOR_GRP(1, 13):
				case MODECHANGES_FOR_GRP(1, 14):
					
					mCurGroup = EXTRACT_GROUP_NUM(byte) - 1;
					mKeyGroups[mCurGroup].mode = (byte >> 1) & 3;
					if (byte & 0x80)	//no more config provided
						mTxF(0xba);	//ack it
					else 				//autorepeat group will be assigned
						mState = KeyboardWaitAutorepeatGroupNum;
					break;
			
			#undef MODECHANGES_FOR_GRP
				
				case 0x78:		//set autorepeat group 0..3 settings
				case 0x7a:
				case 0x7c:
				case 0x7e:
					mCurGroup = (byte >> 1) & 3;
					mState = KeyboardWaitAutorepeatConfigByte1;
					break;
				
				default:
					fprintf(stderr, "kbd RXed 0x%02x\r\n", byte);
					break;
			}
			break;
		
		case KeyboardWaitLedData:				//byte is led data (low 4 bits)
		case KeyboardWaitClickVolumeByte:		//byte is volume (low 3 bits)
		case KeyboardWaitBellVolumeByte:		//byte is volume (low 3 bits)
			mState = KeyboardIdle;
			break;
		
		case KeyboardWaitAutorepeatGroupNum:	//low 2 bits is group to assign
			mKeyGroups[mCurGroup].autorepeatGroup = byte & 3;
			mTxF(0xba);
			mState = KeyboardIdle;
			break;
		
		case KeyboardWaitAutorepeatConfigByte1:	//low 6 bits: timeout vlaue in 5ms increments
			mState = KeyboardWaitAutorepeatConfigByte2;
			mAutorepeatGroups[mCurGroup].timeout = byte & 0x7f;
			break;
		
		case KeyboardWaitAutorepeatConfigByte2:	//low 6 bits: presses per second 
			mState = KeyboardIdle;
			mAutorepeatGroups[mCurGroup].rate = byte & 0x7f;
			break;
		
		case KeyboardLocked:
			if (byte == 0x8b)					//unlock?
				mState = KeyboardIdle;
			break;
	}
}

static uint_fast8_t lk401prvKeyToGroup(enum Lk401Key k)
{
	static const uint8_t groupBoundaries[14][2] = {
		#define GRPENTRY(x)	[x - 1] = {Lk201_GRP ## x ## _first, Lk201_GRP## x ##_last}
			GRPENTRY(1),
			GRPENTRY(2),
			GRPENTRY(3),
			GRPENTRY(4),
			GRPENTRY(5),
			GRPENTRY(6),
			GRPENTRY(7),
			GRPENTRY(8),
			GRPENTRY(9),
			GRPENTRY(10),
			GRPENTRY(11),
			GRPENTRY(12),
			GRPENTRY(13),
			GRPENTRY(14),
		#undef GRPENTRY
	};
	uint_fast8_t i;
	
	for (i = 0; i < 14; i++) {
		
		if (k >= groupBoundaries[i][0] && k <= groupBoundaries[i][1])
			return i + 1;
	}
	
	return 0;
}

void lk401KeyState(enum Lk401Key k, bool down)
{
	uint_fast8_t group = lk401prvKeyToGroup(k);
	uint_fast8_t keyIdx = k, i;
	bool curDown = (mCurStates[keyIdx / 8] >> (keyIdx & 7)) & 1;
	
	if (!group)
		return;
	
	if (curDown == down)
		return;
	
	if (curDown) {	//going up
		
		mKeyGroups[group - 1].nKeysDown--;
		mCurStates[keyIdx / 8] &=~ (1 << (keyIdx & 7));
		
		if (mKeyGroups[group - 1].mode == KeyGroupModeUpDown) {
			
			if (mState != KeyboardLocked) {
				
				//if any other up/down key is held, we need to send the key code of the released key, else we need to send 0xb3
				
				for (i = 0; i < 14; i++) {
					
					if (mKeyGroups[i].mode == KeyGroupModeUpDown && mKeyGroups[i].nKeysDown)
						break;
				}
				
				mTxF(i == 14 ? 0xb3 : keyIdx);
			}
		}
		
	}
	else {			//going down
		
		mKeyGroups[group - 1].nKeysDown++;
		mCurStates[keyIdx / 8] |= (1 << (keyIdx & 7));
		
		if (mState != KeyboardLocked)
			mTxF(keyIdx);	//send it
	}
}


